from __future__ import annotations

from datetime import datetime, timezone
from enum import Enum

from sqlmodel import Field, SQLModel


class ProcessingStatus(str, Enum):
    PENDING = "pending"
    PROCESSING = "processing"
    COMPLETED = "completed"
    FAILED = "failed"


class Article(SQLModel, table=True):

    __tablename__ = "articles"

    id: int | None = Field(default=None, primary_key=True)
    url: str = Field(index=True, unique=True, max_length=500)
    title: str = Field(max_length=300)
    raw_text: str = Field(description="Исходный wiki-текст")
    simplified_text: str | None = Field(
        default=None,
        description="Упрощённый текст для школьников",
    )
    status: ProcessingStatus = Field(default=ProcessingStatus.PENDING)
    error_message: str | None = Field(default=None, max_length=1000)
    token_count_raw: int | None = Field(
        default=None, description="Количество токенов в исходном тексте"
    )
    token_count_simplified: int | None = Field(
        default=None,
        description="Количество токенов в упрощённом тексте",
    )
    processing_time_seconds: float | None = Field(default=None)
    created_at: datetime = Field(default_factory=lambda: datetime.now(timezone.utc))
    updated_at: datetime | None = Field(default=None)

    def mark_processing(self) -> None:
        self.status = ProcessingStatus.PROCESSING
        self.updated_at = datetime.now(timezone.utc)

    def mark_completed(
        self,
        simplified_text: str,
        token_count_raw: int,
        token_count_simplified: int,
        processing_time: float,
    ) -> None:
        self.simplified_text = simplified_text
        self.token_count_raw = token_count_raw
        self.token_count_simplified = token_count_simplified
        self.processing_time_seconds = processing_time
        self.status = ProcessingStatus.COMPLETED
        self.error_message = None
        self.updated_at = datetime.now(timezone.utc)

    def mark_failed(self, error_message: str) -> None:
        self.status = ProcessingStatus.FAILED
        self.error_message = error_message[:1000]
        self.updated_at = datetime.now(timezone.utc)


class ArticleCreate(SQLModel):
    url: str
    title: str
    raw_text: str


class ArticleRead(SQLModel):
    id: int
    url: str
    title: str
    raw_text: str
    simplified_text: str | None
    status: ProcessingStatus
    token_count_raw: int | None
    token_count_simplified: int | None
    created_at: datetime
