from typing import Any

import aiosqlite
import structlog

from ..models import Article, ArticleCreate, ProcessingStatus
from .database import DatabaseService

logger = structlog.get_logger()


class ArticleRepository:

    def __init__(self, db_service: DatabaseService) -> None:
        self.db_service = db_service
        self.logger = structlog.get_logger().bind(repository="article")

    async def create_article(self, article_data: ArticleCreate) -> Article:
        existing = await self.get_by_url(article_data.url)
        if existing:
            raise ValueError(f"Статья с URL {article_data.url} уже существует")

        article = Article(
            url=article_data.url,
            title=article_data.title,
            raw_text=article_data.raw_text,
            status=ProcessingStatus.PENDING,
        )

        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                """
                INSERT INTO articles (url, title, raw_text, status, created_at)
                VALUES (?, ?, ?, ?, ?)
                """,
                (
                    article.url,
                    article.title,
                    article.raw_text,
                    article.status.value,
                    article.created_at,
                ),
            )
            await conn.commit()

            article.id = cursor.lastrowid

        self.logger.info("Статья создана", article_id=article.id, url=article.url)
        return article

    async def get_by_id(self, article_id: int) -> Article | None:
        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                "SELECT * FROM articles WHERE id = ?",
                (article_id,),
            )
            row = await cursor.fetchone()

        if not row:
            return None

        return self._row_to_article(row)

    async def get_by_url(self, url: str) -> Article | None:
        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                "SELECT * FROM articles WHERE url = ?",
                (url,),
            )
            row = await cursor.fetchone()

        if not row:
            return None

        return self._row_to_article(row)

    async def update_article(self, article: Article) -> Article:
        if not article.id:
            raise ValueError("ID статьи не может быть None для обновления")

        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                """
                UPDATE articles SET
                    title = ?,
                    raw_text = ?,
                    simplified_text = ?,
                    status = ?,
                    error_message = ?,
                    token_count_raw = ?,
                    token_count_simplified = ?,
                    processing_time_seconds = ?,
                    updated_at = ?
                WHERE id = ?
                """,
                (
                    article.title,
                    article.raw_text,
                    article.simplified_text,
                    article.status.value,
                    article.error_message,
                    article.token_count_raw,
                    article.token_count_simplified,
                    article.processing_time_seconds,
                    article.updated_at,
                    article.id,
                ),
            )
            await conn.commit()

            if cursor.rowcount == 0:
                raise ValueError(f"Статья с ID {article.id} не найдена")

        self.logger.info("Статья обновлена", article_id=article.id, status=article.status)
        return article

    async def get_articles_by_status(
        self, status: ProcessingStatus, limit: int | None = None
    ) -> list[Article]:
        query = "SELECT * FROM articles WHERE status = ?"
        params: tuple[Any, ...] = (status.value,)

        if limit:
            query += " LIMIT ?"
            params = params + (limit,)

        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(query, params)
            rows = await cursor.fetchall()

        return [self._row_to_article(row) for row in rows]

    async def get_pending_articles(self, limit: int | None = None) -> list[Article]:
        return await self.get_articles_by_status(ProcessingStatus.PENDING, limit)

    async def count_by_status(self, status: ProcessingStatus) -> int:
        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                "SELECT COUNT(*) FROM articles WHERE status = ?",
                (status.value,),
            )
            result = await cursor.fetchone()

        return result[0] if result else 0

    async def get_all_articles(self, limit: int | None = None, offset: int = 0) -> list[Article]:
        query = "SELECT * FROM articles ORDER BY created_at DESC"
        params: tuple[Any, ...] = ()

        if limit:
            query += " LIMIT ? OFFSET ?"
            params = (limit, offset)

        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(query, params)
            rows = await cursor.fetchall()

        return [self._row_to_article(row) for row in rows]

    async def delete_article(self, article_id: int) -> bool:
        async with self.db_service.get_connection() as conn:
            cursor = await conn.execute(
                "DELETE FROM articles WHERE id = ?",
                (article_id,),
            )
            await conn.commit()

        deleted = cursor.rowcount > 0
        if deleted:
            self.logger.info("Статья удалена", article_id=article_id)

        return deleted

    def _row_to_article(self, row: aiosqlite.Row) -> Article:
        return Article(
            id=row["id"],
            url=row["url"],
            title=row["title"],
            raw_text=row["raw_text"],
            simplified_text=row["simplified_text"],
            status=ProcessingStatus(row["status"]),
            error_message=row["error_message"],
            token_count_raw=row["token_count_raw"],
            token_count_simplified=row["token_count_simplified"],
            processing_time_seconds=row["processing_time_seconds"],
            created_at=row["created_at"],
            updated_at=row["updated_at"],
        )
